#!/usr/bin/env Rscript
#############################################################################################
# This script is used  as a library to create a Forest management and a Replanting strategy #
# NETCDF map of Europe in which each pixel contains the optimal scenario                    # Author: Guillaume Marie                                                                   #
#############################################################################################

############################################ SETUP #####################################

library(grid)
library(gridExtra)
library(ncdf)
library(ncdf4)
library(maps)
library(raster)
library(reshape2)
library(rpart)
library(data.table)

###################### FUNCTIONS ###################################################

# This function is used to filter the choosen strategy given a thresold
# if the gain of the new strategy over the BAU is inferior to the thresold
# then BAU is conservedwill be selected
filt=function(data,no,ths,target){
    best=NULL
    for(i in 1:length(data[,target])){
        if(is.na(data[i,'best'])){
            best[i]=NA
        }
        else{
            if(abs(data[i,data[i,'best']+2]-data[i,target])<ths) best[i]=1
            else best[i]=data[i,'best']
        }
    }
    data[,no]=best
    return(data)
}

# This function is used to create a Forest management strategy
# NETCDF map of Europe in which each pixel contains the optimal
# strategy for a objective of the portfolio
remap=function(Op,map,target){
    nc=nc_open(map, write=TRUE)
    hig=nc_open('/ccc/work/cont003/dofoco/dofoco/MAPS/FM/FUTURE/RGRID/FM_desired_2.nc')
    con=nc_open('/ccc/work/cont003/dofoco/dofoco/MAPS/FM/FUTURE/RGRID/FM_desired_1.nc')
    cop=nc_open('/ccc/work/cont003/dofoco/dofoco/MAPS/FM/FUTURE/RGRID/FM_desired_3.nc')
    cur=nc_open('/ccc/work/cont003/dofoco/dofoco/MAPS/FM/RGRID/fm_strat_map_2010_mean.nc')
    nc.x=ncvar_get(nc,'FM_STRAT');nc.con=ncvar_get(con,'FM_STRAT'); nc.cur=ncvar_get(cur,'FM_STRAT')
    nc.cop=ncvar_get(cop,'FM_STRAT');nc.hig=ncvar_get(hig,'FM_STRAT')
    for( p in 1:28){
        nnc=matrix(0,128,119)
        for(x in 1:128){
           for (y in 1:119){
                Nsenar=subset(Op, X==x &  Y==y, target)
                if(is.na(Nsenar)) nnc[x,y]=nc.cur[x,y,p]
                else {
                    if(Nsenar==2) nnc[x,y]=nc.con[x,y,p]
                    else {
                        if ((Nsenar==3 |Nsenar==4 | Nsenar==5)) nnc[x,y]=nc.hig[x,y,p]
                            else {
                                if (Nsenar==6) { nnc[x,y]=nc.cop[x,y,p] }
                                else {
                                    if(Nsenar==1) nnc[x,y]=nc.cur[x,y,p]
                                    else print(Nsenar)
                                }
                            }
                    }
                }
           }
       }
       ncvar_put(nc, "FM_STRAT", nnc, start=c(1,1,p), count=c(-1,-1,1))
       print(paste0('PFT',p,'is finished'))
    }
    nc_close(nc)
    return(print('file remaped'))
}

# This function is used to create a species replanting strategy
# NETCDF map of Europe in which each pixel givecontains the optimal 
# strategy for the objective of the portfolio
remap.SP=function(Op,OpMap,target){
    nc=nc_open(OpMap, write=TRUE)
    C=nc_open('/ccc/work/cont003/dofoco/dofoco/MAPS/FM/FUTURE/RGRID/coniferous_rgrid.nc')
    D=nc_open('/ccc/work/cont003/dofoco/dofoco/MAPS/FM/FUTURE/RGRID/deciduous_rgrid.nc')
    nc.x=ncvar_get(nc,'NEWSPECIES')
    nc.C <- ncvar_get(C,'NEWSPECIES');nc.D <-ncvar_get(D,'NEWSPECIES')
    pdf(paste0('maps_',target,'_replant.pdf'),onefile=TRUE)
    for( p in 1:28){
        mnc.C <- melt(nc.C[,,p]);  mnc.D <- melt(nc.D[,,p])
        nnc=matrix(NaN,128,119)
        for(x in 1:128){
            for (y in 1:119){
                Nsenar <- subset(Op, X==x &  Y==y, target)
                if(is.na(Nsenar)) nnc[x,y]= NaN
                else {
                    if(Nsenar==1 | Nsenar==2 | Nsenar==3) nnc[x,y]=p
                    else {
                        if (Nsenar==4) nnc[x,y]=nc.C[x,y,p]
                        else {
                            if (Nsenar==5 | Nsenar==6) nnc[x,y]=nc.D[x,y,p]
                            else print(Nsenar)
                        }
                    }
                }
            }
       }
       ncvar_put(nc, "NEWSPECIES", nnc, start=c(1,1,p), count=c(-1,-1,1))
       print(paste0('PFT',p,'is finished'))
   }
   dev.off()
   nc_close(nc)
   return(print('file remaped'))
}

# This function is used to find the min or the max of one target variable and for each pixel
# ref = reference simulation (BWAC, CWAC2017, CBAC2017)
# sim = a group of simulation which represent the strategy (CWA1,CWD2,....etc)
# an = you can average the target variable over year (begin year , end year) 
# var = the target variable(alb_vis, atmco2, t2m)
# ths = thresold below which the best strategy is always BAU
# method = min or max target variable
# sour = type of output file (SRF, ATM, SBG)
# hash = how the output files are cut (YE, MO, DA)
Optimize.MA <- function(ref,sim,an,var,ths,method,sour,hash){
    simu <- c(ref,sim)
    func <- get(method)
    trash <- list()
    for(i in simu) trash[[i]] <- read.table(paste0(sour,'_',i,'_',hash,'.test'),header=TRUE)
    if(hash=='MO') land <- melt(trash,id=c("X","Y","an","mo"))
    else land <- melt(trash,id=c("X","Y","an"))
    land <- subset(land,an<=an[2] &  an>=an[1] & variable==var)
    land <- dcast(land, X + Y ~ L1, value.var='value', mean)
    land <- land[,c('X','Y',simu)]
    land[,simu[1]] <- ifelse(is.na(land[,simu[1]]),0,land[,simu[1]])
    land[,'CDdiff'] <- ifelse(abs(land[,simu[6]]-land[,simu[5]])<ths,0,1)
    land[,'best'] <- apply(land[,simu],1,func)
    land[,'best'] <- ifelse(is.na(land[,simu[2]]),NA,land[,'best'])
    land[,'best_Esp']=ifelse(land[,'best']==4 |land[,'best']==9|land[,'best']==10,'Change to Conifer',NA)
    land[,'best_Esp']=ifelse(land[,'best']==5 |land[,'best']==6|land[,'best']==7 | land[,'best']==8,'Change to Decidious',land[,'best_Esp'])
    land[,'best_Esp']=ifelse(land[,'best']==1 |land[,'best']==2|land[,'best']==3,'No change',land[,'best_Esp'])
    land[,'best_Fm']=ifelse(land[,'best']==2,'Change to Unmanaged',NA)
    land[,'best_Fm']=ifelse(land[,'best']==1,'No Change',land[,'best_Fm'])
    land[,'best_Fm']=ifelse(land[,'best']==3 | land[,'best']==4 | land[,'best']==5,'Change to Highstand',land[,'best_Fm'])
    land[,'best_Fm']=ifelse(land[,'best']==6,'Change to Coppice',land[,'best_Fm'])
    land[,'best_Fm']=ifelse(land[,'best']==8| land[,'best']==10,'Change to FastRot',land[,'best_Fm'])
    land[,'best_Fm']=ifelse(land[,'best']==7| land[,'best']==9,'Change to LowDens',land[,'best_Fm'])
    land1=filt(land,'best1',ths=ths,simu[1])
    land1[,'best1']=ifelse(land1[,'CDdiff']==0 & land1[,'best1']==6,5,land1[,'best1'])
    land1[,'best2']=ifelse(is.na(land1[,'best1']),1,land1[,'best1'])
    land1[,simu[1]]=ifelse(land1[,simu[1]]==0,NA,land[,simu[1]])
    return(land1)
}

########################### END FUNCTIONS ##################################################




