#!/usr/bin/env python

#-------------------------------------------------------------------
# Objectives
#-------------------------------------------------------------------
# This script calculates the carbon emmitted during forest management
# and the carbon absorbed during product use and product substitution.
# Sink and sources are adjusted for pressure balance between the 
# ocean-atmosphere

# This script was developed for future simulations but only uses
# present day emission factors. The justification for this is that
# we will compare scenarios and so if an emission factor decreases
# it will decrease in all scenarios. Nevertheless, changes in EF
# could changes the outcome (and the ranking of the scenarios)

# The simulations did not change the fraction of agricultural land 
# so we can ignore it in the analysis.

# nfm_types = 6     Number of forest management strategies
# ifm_none = 1      No human intervention in the forests.
# ifm_thin = 2      Regular thinning and harvesting of wood based on RDI.
# ifm_cop = 3       Coppicing for fuelwood.
# ifm_src = 4       Short rotation coppices for biomass production.
# ifm_crop = 5      Crop harvest
# ifm_grass = 6     Grazing or cutting 
#
# ncut_times = 9    Number of reasons for tree cutting and wood harvested.
# icut_clear = 1    A clearcut where all biomass is removed.
# icut_thin = 2     Thinning of biomass to reduce the number of trees.
# icut_lcc_wood = 3 Wood harvest following land cover change (LCC)
# icut_lcc_res = 4  Site clearing, removal of the stumps and branches LCC
# icut_crop = 5     Crop harvest
# icut_grass = 6    Grazing or cutting
# icut_cop1 = 7     The first coppice cut
# icut_cop2 = 8     The second (and subsequent) coppice cut
# icut_cop3 = 9     The last coppice cut (only for SRC)

# Author: Sebastiaan Luyssaert
 
#-------------------------------------------------------------------
# Load modules and functions Definitions
#-------------------------------------------------------------------
import time
from math import exp
import numpy as np
import scipy as sc
import matplotlib.pyplot as plt
import shelve
from Scientific.IO.NetCDF import NetCDFFile as Dataset
np.set_printoptions(threshold='nan')
from operator import add
import sys


#----------------------------------------------------------------
# Simulation settings
#----------------------------------------------------------------
# Output files
jobname = ['CBAC']
#jobname = ['CBAC','CBA1','CBA2','CBC2','CBD2','CBD3','CBD5','CBD6','CBC5','CBC6']
#jobname = ['CWAC','CWA1','CWA2','CWC2','CWD2','CWD3','CWD5','CWD6','CWC5','CWC6']
#jobname = ['CWAC','CWA1','CWA2','CWC2','CWD2','CWD3']
#jobname = [sys.argv[1]]
loc='store'
path_countries = '/ccc/work/cont003/dofoco/dofoco/MAPS/LAND_MASK/'
year_first = 2011
year_last = 2015
recalculate='y'

if (jobname[0]=='CBAC'):
    suffix = 'B'
elif (jobname[0]=='CWAC'):
    suffix = 'W'
else:
    print 'error: suffix not known'
#endif

#----------------------------------------------------------------
# Aggregation settings
#----------------------------------------------------------------
n_regions = 13
file_region = 'sociobioclimatic_regions_zoom.nc'
agg_first = 2001
agg_last = 2100

#----------------------------------------------------------------
# EU-wide emision factors
#----------------------------------------------------------------
# kWh_to_J
kWh_to_J = 3600000.
# CO2 to carbon
CO2_to_C = 12./44.
# ha to m-2
ha_to_m2 = 1./10000.
# kg to g
kg_to_g = 1000.
# G to unit
G_to_unit = 1000000000.
# ton to unit
ton_to_unit = 1000000.
# Carbon in biomass (gC gDM-1)
carbon_to_biomass = 0.5
# Transport distance roundwood (km)
transport_roundwood = 100.
# Transport distance biomass (km)
transport_biomass = 50.
# Transport by truck (kg CO2 tkm-1 -> gC km-1) with big
# trucks values from https://www.gov.uk/government/uploads/
# system/uploads/attachment_data/file/218576/
# lct-steering-group-carbon-em.xls
EF_trucking = 1.120*CO2_to_C*kg_to_g/ton_to_unit/carbon_to_biomass
# Mechanised harvest (kg CO2 ha-1 -> gC m-2)
EF_harvest = 233.*CO2_to_C*ha_to_m2*kg_to_g
# Mechanised planting (kg CO2 ha-1 -> gC m-2)
EF_planting = 93.*CO2_to_C*ha_to_m2*kg_to_g
# Mechanised thinning (kg CO2 ha-1 -> gC m-2)
EF_thinning = 69.*CO2_to_C*ha_to_m2*kg_to_g
# Product substitution (Roger Sathre and Jennifer O Connor 2010)
# Assumes products are land-filled at the end of their life
product_substitution = 1.1
# Not all biomass will be used for bio-energy (Assumed)
wood_to_energy = 0.5
# CHP (GJ t-1 DM -> J gC-1) (Liberloo et al 2010)
energy_from_CHP = 6.7 / carbon_to_biomass * G_to_unit / ton_to_unit
# emissions from CHP gC gC-1 (Assumed for burning,drying and chipping)
# This is the real emission but if you this value
# we should use NEP instead of NBP (or account in
# different way for harvest)
emissions_from_CHP = 1.05
# emissions from CHP gC gC-1 (Assumed for drying and chipping)
# This is the emission assuming the burned biomass
# will be used for plant growth again. This is a classic
# assumption in LCA but overlooks the fact that it may 
# take decennia to centuries before this happens. 
# emissions_from_CHP = 0.05  


#----------------------------------------------------------------
# Components for in-situ and products
#----------------------------------------------------------------
in_situ = ['LITTER_STR_AB','LITTER_MET_AB','LITTER_STR_BE',\
              'LITTER_MET_BE','LITTER_WOD_AB','LITTER_WOD_BE',\
              'LEAF_M','SAP_M_AB','SAP_M_BE','HEART_M_AB',\
              'HEART_M_BE','ROOT_M','FRUIT_M','RESERVE_M',\
              'LABILE_M','CARBON_ACTIVE','CARBON_SLOW',\
              'CARBON_PASSIVE'] 
n_in_situ = len(in_situ)
nep_flux = ['GPP','GROWTH_RESP','MAINT_RESP','HET_RESP']
n_nep_flux = len(nep_flux)
# The ORCHIDEE fluxes are written as the daily average
# they need to be multiplied with 365 to obtain the annual
# value.
dt = 365

# PFT numbers
tree_min = 1
tree_max = 58


#----------------------------------------------------------------
# Country map
#----------------------------------------------------------------
# Define country list
# (1) Country name, (2) country number, (3) GHG emissions for current 
# energy production (g CO2 eq kWh-1)
country = [\
    ['Albania',2.,1020.],
    ['Andorra',5.,810.],
    ['Austria',13.,777.],
    ['Belarus',19.,1020.],
    ['Belgium',20.,687.],
    ['Bosnia and Herzegovina',26.,1017.],
    ['Bulgaria',32.,981.],
    ['Croatia',51.,812.],
    ['Cyprus',53.,868.],
    ['Czech Republic',54.,1010.],
    ['Denmark',55.,904.],
    ['Estonia',64.,1014.],
    ['Finland',69.,853.],
    ['France',70.,810.],
    ['Germany',75.,927.],
    ['Greece',78.,894.],
    ['Hungary',89.,780.],
    ['Iceland',90.,868.],
    ['Ireland',95.,766.],
    ['Italy', 97.,744.],
    #['Kosovo',xx,1020.],
    ['Latvia',109.,615.],
    ['Liechtenstein',114.,777.],
    ['Lithuania',115.,591.],
    ['Luxembourg',116.,614.],
    ['Macedonia',117.,1020.],
    ['Malta',123.,868.],
    ['Moldova',130.,1020.],
    ['Monaco',131.,810.],
    #['Montenegro',xx,1020.],
    ['Netherlands',140.,748.],
    ['Norway',149.,641.],
    ['Poland',159.,1000.],
    ['Portugal',160.,840.],
    ['Romania',164.,907.],
    ['Serbia and Montenegro',172.,1012.],
    ['Slovakia',176.,842.],
    ['Slovenia',177.,982.],
    ['Spain',182.,797.],
    ['Sweden',192.,857.],
    ['Switzerland',193.,628.],
    ['Ukraine',209.,1020.],
    ['United Kingdom',211.,854.]]
#country = [['France',70.,810.]]

# Read country map
file_name = 'global_country_mask_zoom.nc'
fullfile_name = path_countries+file_name
dummy = Dataset(fullfile_name,'r')
lon_country = dummy.variables['lon'][:]
lat_country = dummy.variables['lat'][:]
n_country = np.shape(country)[0]
c_temp = dummy.variables['CNID'][:,:]*1.
dummy.close()


#----------------------------------------------------------------
# Open the first file to extract some dimensions 
#----------------------------------------------------------------
#file_name = jobname[0] + '_' + str(year_first) +'0101_' + \
#    str(year_first) + '1231_1Y_stomate_history.nc'
#path_sim = '/ccc/scratch/cont003/dsm/p529luy/IGCM_OUT/OL2/PROD/' + \
#    'SECSTO/'+jobname[0]+'/SBG/Output/YE/'
file_name = 'CWAC2017_' + str(year_first) +'0101_'+ str(year_first) + '1231_1Y_stomate_history.nc'
path_sim = '/ccc/store/cont003/dsm/p529luy/IGCM_OUT/OL2/PROD/SECSTO/CWAC2017/SBG/Output/YE/'
fullfile_name = path_sim+file_name
dummy = Dataset(fullfile_name, mode='r')
lon_sim = dummy.variables['lon'][:]
lat_sim = dummy.variables['lat'][:]
nvm = len(dummy.variables['PFT'][:])
n_years = (year_last-year_first+1)
n_jobs = len(jobname)
n_lat = max(len(lat_country),len(lat_sim))
n_lon = max(len(lon_country),len(lon_sim))
# Read here (so we can close the nc), used in the country mask
temp = dummy.variables['VEGET_MAX'][:,:,:]*1.
dummy.close()

print len(temp)

#----------------------------------------------------------------
# Check dimensions 
#----------------------------------------------------------------
for i_lat in range(0,n_lat):
    if (lat_country[i_lat]!=lat_sim[i_lat]):
        print 'ERROR: country map and simul have a different lat'        
for i_lon in range(0,n_lon):
    if (lon_country[i_lon]!=lon_sim[i_lon]):
        print 'ERROR: country map and simul have a different lon'        
if (n_years<=0):
    print 'ERROR: check start and end year'
    

#----------------------------------------------------------------
# Make a map of the area selected in the variable country
#----------------------------------------------------------------
c_map = np.zeros(shape=(n_lat,n_lon),dtype=np.int)
current_production_emissions = np.zeros(shape=(n_lat,n_lon))
for i_country in range(0,n_country):
	index = np.where(c_temp==country[i_country][1])
        c_map[index[0],index[1]]=1 
        #Convert from kWh-1 to J
        current_production_emissions[index[0],index[1]] = country[i_country][2]*CO2_to_C/kWh_to_J
# endfor

# There are some small inconsistencies between the ORCHIDEE
# grid and the country mask. Remove inconsistencies.
c_map2 = np.ones(shape=(n_lat,n_lon),dtype=np.int)
index = np.where(temp[0,0,:,:]==9.96920997e+36)
print len(index[0])
c_map2[index[0],index[1]]=0
c_map3=np.add(c_map,c_map2)

index = np.where(c_map3==1)
current_production_emissions[index[0],index[1]] = country[2][2]*CO2_to_C/kWh_to_J

c_map3=np.divide(c_map3,c_map3)

#print c_map3


ncfile = Dataset('LCA_mask.nc','w')
ncfile.createDimension('latitude',n_lat)
ncfile.createDimension('longitude',n_lon)
lats = ncfile.createVariable('latitude','f',('latitude',))
lons = ncfile.createVariable('longitude','f',('longitude',))
lats.axis = 'Y'
lats.units = 'degrees north'
lons.axis = 'X'
lons.units = 'degrees east'
lats[:] = lat_sim
lons[:] = lon_sim
mask = ncfile.createVariable('countries','i',('latitude','longitude'))
mask2 = ncfile.createVariable('countries2','i',('latitude','longitude'))
for i in range(0,n_lat):
    for j in range(0,n_lon):
        mask[i,j] = c_map[i,j]
        mask2[i,j] = c_map3[i,j]
    # endfor
# endfor
mask.units='selected countries (0 or 1)'
mask2.units='selected countries (0 or 1)'
ncfile.close()


#----------------------------------------------------------------
# Initialize variables
#----------------------------------------------------------------
harvest_pool = np.zeros(shape=(nvm,n_lat,n_lon))
harvest_cut = np.zeros(shape=(nvm,n_lat,n_lon))
harvest_area = np.zeros(shape=(nvm,n_lat,n_lon))
veget_max = np.zeros(shape=(nvm,n_lat,n_lon))
area = np.zeros(shape=(n_lat,n_lon))
regions = np.zeros(shape=(n_lat,n_lon))
forest_area = np.zeros(shape=(n_lat,n_lon))
bio_wood = np.zeros(shape=(n_lat,n_lon))
electricity = np.zeros(shape=(n_lat,n_lon)) 
nep = np.zeros(shape=(n_jobs,n_lat,n_lon))
emission_fm = np.zeros(shape=(n_jobs,n_lat,n_lon))
subs_wood = np.zeros(shape=(n_jobs,n_lat,n_lon))
subs_energy = np.zeros(shape=(n_jobs,n_lat,n_lon))
#LCA = np.zeros(shape=(n_jobs,n_lat,n_lon))
veget_max = np.zeros(shape=(nvm,n_lat,n_lon))
area = np.zeros(shape=(n_lat,n_lon))
resid = np.zeros(shape=(n_jobs,n_years,n_years,n_lat,n_lon))
atm_co2 =  np.zeros(shape=(n_jobs,n_lat,n_lon))
nbp = np.zeros(shape=(n_jobs,n_years,n_lat,n_lon))
uptake_nbp = np.zeros(shape=(n_jobs,n_years,n_lat,n_lon))
#atm_region = np.zeros(shape=(n_jobs,n_regions))

#--------------------------------------------------------------
# Discount the NBP for ocean uptake
#--------------------------------------------------------------
# Calculate how much carbon stayed in the atmosphere
# This is done with a decay function to account for 
# the amount of carbon that is taken up by the ocean
        
# Calculate the decay rate
# R_decay = 0.70211*exp(-0.35.*t) + 0.01341*exp(-0.05.*t) 
#  - 0.7185*exp(-0.45833.*t) + 0.002932*exp(-0.01.*t)
#  where t=year

# R_decay = [0 R_decay]
time = np.arange(0, year_last-year_first+1, dtype=np.integer)

# Calculate the decay coefficients
R_decay=np.empty(shape=(n_years))
for i_year in range (1,n_years):
    R_decay[i_year] = 0.70211*exp(-0.35*i_year) + \
        0.01341*exp(-0.05*i_year) - \
        0.7185*exp(-0.45833*i_year) + \
        0.002932*exp(-0.01*i_year)
#endfor

R_decay[0]=0

  
#----------------------------------------------------------------
# Replot only or process & plot
#----------------------------------------------------------------
if recalculate=='n':

    #load an array that was build during a 
    #previous session
    store = shelve.open('archived_maps')
    fm_strat_map = store['archive']
    store.close()

else:

    for i_year in range(0,n_years):

        # Initialize
        nep[:,:,:] = 0.
        emission_fm[:,:,:] = 0.
        subs_wood[:,:,:] = 0.
        subs_energy[:,:,:] = 0.
        #LCA[:,:,:] = 0.
        atm_co2[:,:,:] = 0.

        for i_job in range(0,n_jobs):

            # Open simulations
            year = year_first+i_year
            path_sim = '/ccc/' + loc + '/cont003/dsm/'+\
                'p529luy/IGCM_OUT/OL2/PROD/SECSTO/'\
                +jobname[i_job]+'/SBG/Output/YE/'
            file_name = \
                jobname[i_job] + '_' + str(year) +'0101_' + \
                str(year) + '1231_1Y_stomate_history.nc'
            fullfile_name = path_sim + file_name
            print file_name
            #---------------------------------------------------
            # Calculate emissions from forest management
            #---------------------------------------------------
            # Read all harvest related variables
            # Dimensions (pft,lat,lon)
            dummy_map = Dataset(fullfile_name,'r')
            temp = dummy_map.variables['HARVEST_POOL_C'][:,:,:]*1.
            harvest_pool[:,:,:] = temp[:,:,:]
            #dummy_map.close()
            # The harvest cut and area are 0 for 364 days. On day 365 
            # the harvest is calculated but the daily average is written
            # to the history files.
            #dummy_map = Dataset(fullfile_name,'r')
            temp = dummy_map.variables['HARVEST_CUT'][:,:,:]*365.
            harvest_cut[:,:,:] = temp[:,:,:]
            #dummy_map.close()
            #dummy_map = Dataset(fullfile_name,'r')
            temp = dummy_map.variables['HARVEST_AREA'][:,:,:]*365.
            harvest_area[:,:,:] = temp[:,:,:]
            dummy_map.close()
            # Mask the data (pft by pft)
            for ivm in range(0,nvm):
                harvest_pool[ivm,:,:] = harvest_pool[ivm,:,:,] * c_map3[:,:] 
                harvest_cut[ivm,:,:] = harvest_cut[ivm,:,:] * c_map3[:,:]
                harvest_area[ivm,:,:] = harvest_area[ivm,:,:] * c_map3[:,:] 
            #endfor ivm            
            for i_lat in range(0,n_lat):
                for i_lon in range(0,n_lon):
                    # Follow the sign convention (+ is an emission)
                    if c_map3[i_lat,i_lon]==0:
                        emission_fm[i_job,i_lat,i_lon]=0.
                    else:
                        # The pixel falls within our domain
                        # now loop over the PFTs. The variable
                        # harvest_cut helps to exclude grasses and
                        # crops
                        for ivm in range(0,nvm):                        
                            if (harvest_cut[ivm,i_lat,i_lon]==0 and \
                                    harvest_cut[ivm,i_lat,i_lon]==5 and \
                                    harvest_cut[ivm,i_lat,i_lon]==6):       
                                emission_fm[i_job,i_lat,i_lon]=0.
                            elif harvest_cut[ivm,i_lat,i_lon]==1:
                                # Round wood: transport+harvest+replanting
                                # Transport (gC * km * gC gC-1 km-1)
                                # Harvest (m2 * gC m-2)
                                # Replanting  (m2 * gC m-2) 
                                emission_fm[i_job,i_lat,i_lon] = \
                                    emission_fm[i_job,i_lat,i_lon] + \
                                    (harvest_pool[ivm,i_lat,i_lon] * \
                                    EF_trucking * transport_roundwood) + \
                                    (harvest_area[ivm,i_lat,i_lon]*EF_harvest)+\
                                    (harvest_area[ivm,i_lat,i_lon]*EF_planting)
                            elif harvest_cut[ivm,i_lat,i_lon]==2:
                                # biomass: transport+thinning
                                # Transport (gC * km * gC gC-1 km-1)
                                # Thinning (m2 * gC m-2)
                                emission_fm[i_job,i_lat,i_lon] = \
                                    emission_fm[i_job,i_lat,i_lon] + \
                                    (harvest_pool[ivm,i_lat,i_lon] * \
                                    EF_trucking * transport_biomass) + \
                                    (harvest_area[ivm,i_lat,i_lon]*EF_thinning)
 
                            elif harvest_cut[ivm,i_lat,i_lon]==3:
                                # Round wood: transport+harvest
                                # Transport (gC * km * gC gC-1 km-1)
                                # Harvest (m2 * gC m-2)
                                emission_fm[i_job,i_lat,i_lon] = \
                                    emission_fm[i_job,i_lat,i_lon] + \
                                    (harvest_pool[ivm,i_lat,i_lon] * \
                                    EF_trucking * transport_roundwood) + \
                                    (harvest_area[ivm,i_lat,i_lon]*EF_harvest)

                            elif harvest_cut[ivm,i_lat,i_lon]==4:
                                # biomass: transport + harvest (no replanting)
                                # Transport (gC * km * gC gC-1 km-1)
                                # Harvest (m2 * gC m-2)
                                emission_fm[i_job,i_lat,i_lon] = \
                                    emission_fm[i_job,i_lat,i_lon] + \
                                    (harvest_pool[ivm,i_lat,i_lon] * \
                                    EF_trucking * transport_biomass) + \
                                    (harvest_area[ivm,i_lat,i_lon]*EF_harvest)

                            elif ( harvest_cut[ivm,i_lat,i_lon]==7 and
                                    harvest_cut[ivm,i_lat,i_lon]==8):
                                # biomass: transport + thinning
                                # Transport (gC * km * gC gC-1 km-1)
                                # Thinning (m2 * gC m-2)
                                emission_fm[i_job,i_lat,i_lon] = \
                                    emission_fm[i_job,i_lat,i_lon] + \
                                    (harvest_pool[ivm,i_lat,i_lon] * \
                                    EF_trucking * transport_biomass) + \
                                    (harvest_area[ivm,i_lat,i_lon]*EF_thinning)
    
                            elif harvest_cut[ivm,i_lat,i_lon]==9:
                                #biomass: transport+harvest+replanting
                                # Transport (gC * km * gC gC-1 km-1)
                                # Harvest (m2 * gC m-2)
                                # Replanting  (m2 * gC m-2)
                                emission_fm[i_job,i_lat,i_lon] = \
                                    emission_fm[i_job,i_lat,i_lon] + \
                                    (harvest_pool[ivm,i_lat,i_lon] * \
                                    EF_trucking * transport_biomass) + \
                                    (harvest_area[ivm,i_lat,i_lon]*EF_harvest)+\
                                    (harvest_area[ivm,i_lat,i_lon]*EF_planting)

                            #endif harvest_cut
                        #endfor ivm
                    #endif
                #endfor i_lon
            #endfor i_lat

            # Remove missing values
            index = np.where(emission_fm>1000000000000000000)
            emission_fm[i_job,index[1],index[2]] = 0


         #   #---------------------------------------------------
         #   # Calculate in-situ nbp
         #   #---------------------------------------------------
         #   # If we use the NBP we have to use the assumption
         #   # that the emissions are taken up in the same year.
         #   dummy_map = Dataset(fullfile_name,'r')
         #   veget_max[:,:,:] = dummy_map.variables['VEGET_MAX'][:,:,:]*1.
         #   area[:,:] = (dummy_map.variables['RESOLUTION_X'][:,:]*c_map3[:,:])*\
         #       (dummy_map.variables['RESOLUTION_Y'][:,:]*c_map3[:,:])*1.
         #   dummy_map.close()
            
         #   # Accumulate the in-situ carbon (gC m-2)
         #   for i_in_situ in range(0,n_in_situ): 
         #       dummy_map = Dataset(fullfile_name,'r')
         #       temp = dummy_map.variables[in_situ[i_in_situ]][:,:,:,:]*1.
         #       dummy_map.close()

         #       # Loop over only the forest PFTs
         #       for ivm in range(tree_min,tree_max):
         #           uptake_nbp[i_job,i_year,:,:] = \
         #               uptake_nbp[i_job,i_year,:,:] + \
         #               temp[0,ivm,:,:]*veget_max[ivm,:,:]*\
         #               area[:,:]
         #       # endfor ivm                
         #   # end for i_in_situ

         #   # Remove missing values
         #   index = np.where(uptake_nbp>1000000000000000000)
         #   uptake_nbp[i_job,i_year,index[2],index[3]] = 0 

            #---------------------------------------------------
            # Calculate nep
            #---------------------------------------------------
            # By using nep we no longer have to make the assumption
            # that the emissions from wood-energy are taken up the 
            # same year. It also allows us to calculate the nep
            # for a single year. The latter helps to solve the memory
            # issue we were facing when trying to calculate the whole
            # time series at once.
            # This approach does no longer impose carbon neutrality!
            dummy_map = Dataset(fullfile_name,'r')
            veget_max[:,:,:] = dummy_map.variables['VEGET_MAX'][:,:,:]*1.
            area[:,:] = (dummy_map.variables['RESOLUTION_X'][:,:]*c_map3[:,:])*\
                (dummy_map.variables['RESOLUTION_Y'][:,:]*c_map3[:,:])*1.
            index = np.where(veget_max>1000000000000000000)
            veget_max[index[0],index[1],index[2]] = 0
            dummy_map.close()

            # Accumulate the in-situ carbon (gC m-2)
            # Follow sign convention (+ is an emission)
            for i_nep_flux in range(0,n_nep_flux): 
                dummy_map = Dataset(fullfile_name,'r')
                if (nep_flux[i_nep_flux]=='GPP'):
                    # GPP has a positive sign
                    temp = dummy_map.variables[nep_flux[i_nep_flux]][:,:,:,:]*-1.
                else:
                    # Respiration has a negative sign
                    temp = dummy_map.variables[nep_flux[i_nep_flux]][:,:,:,:]*1.
                #endif
                dummy_map.close()
    
                # Loop over only the forest PFTs
                for ivm in range(tree_min,tree_max):
                    nep[i_job,:,:] = \
                        nep[i_job,:,:] + \
                        temp[0,ivm,:,:]*veget_max[ivm,:,:]*\
                        area[:,:]*dt
                # endfor ivm                
            # end for i_nep_flux

            # Remove missing values
            index = np.where(nep>1000000000000000000)
            nep[i_job,index[1],index[2]] = 0
            index = np.where(nep<-1000000000000000000)
            nep[i_job,index[1],index[2]] = 0
            
            #---------------------------------------------------
            # Calculate avoided emissions from wood use
            #---------------------------------------------------
            # Only the long-lived wood products have a 
            # substitution value that we account for
            dummy_map = Dataset(fullfile_name,'r')
            temp = dummy_map.variables['PROD_L'][:,:,:]*1.
            dummy_map.close()

            # Fossil fuel substitution from wood product use (gC m-2)
            # Follow sign convention (+ is an emission)
            subs_wood[i_job,:,:] = -1. * temp[0,1,:,:] * \
                product_substitution * c_map3[:,:]
          
            # Remove missing values
            index = np.where(subs_wood<-1000000000000000)
            subs_wood[i_job,index[1],index[2]] = 0


            #---------------------------------------------------
            # Calculate avoided emissions from bioenergy
            #---------------------------------------------------
            # The short-lived products are decomposed
            # within a year so PROD_S is zero. We can
            # back calculate this pool from harvest minus
            # the PROD_M and PROD_L
            for ivm in range(tree_min,tree_max):
                bio_wood[:,:] = bio_wood[:,:] + harvest_pool[ivm,:,:]
            # endfor ivm
            
            # Substract the long lived and medium lived wood 
            # from the total harvest to obtain an estimate
            # of the biomass for bioenergy
            bio_wood[:,:] = bio_wood[:,:] - \
                (temp[0,1,:,:] * c_map3[:,:])

            dummy_map = Dataset(fullfile_name,'r')
            temp = dummy_map.variables['PROD_M'][:,:,:]*1.
            dummy_map.close()
            bio_wood[:,:] = bio_wood[:,:] - \
                (temp[0,1,:,:] * c_map3[:,:])

            # Remove missing values
            index = np.where(bio_wood>1000000000000000)
            bio_wood[index[0],index[1]] = 0.

            # Remove negative harvest values
            index = np.where(bio_wood<0.)
            bio_wood[index[0],index[1]] = 0.

            # Not all the biomass will be used for
            # bioenergy production (gC). 
            bio_wood[:,:] = bio_wood[:,:] * wood_to_energy

            # All the biomass for bioenergy will be used
            # in CHP for electricity production (J)
            electricity[:,:] = bio_wood[:,:] * energy_from_CHP

            # Avoided emissions = Emission from CHP electricity 
            # production (gC) - avoided emissions from CHP (thus
            # compared to the emissions from current electricity
            # production)
            # Follow the sign convention (+ is an emission)
            subs_energy[i_job,:,:] = \
                (bio_wood[:,:] * emissions_from_CHP)\
                - (current_production_emissions[:,:] * \
                electricity[:,:])

         #   #---------------------------------------------------
         #   # Calculate LCA
         #   #---------------------------------------------------
         #   LCA[i_job,:,:] = emission_fm[i_job,:,:] \
         #       +nep[i_job,:,:]\
         #       +subs_wood[i_job,:,:] \
         #       +subs_energy[i_job,:,:]

            #---------------------------------------------------
            # Calculate the residual in the atmosphere
            #---------------------------------------------------
            # Build a matrix to account for the decay 
            # The rows are the years for which LCA has been calculated
            # The columns is the years for which the decay has to
            # be calculated.
            # i_year got its value far above. We fill this matrix
            # year by year.
            for i_withinyear in range (0,n_years):

                if (i_withinyear == i_year):
                    # At the first time step of each year R_decay[0]
                    # has to used.
                    resid[i_job,i_year,i_year,:,:] = \
                        emission_fm[i_job,:,:] \
                        +nep[i_job,:,:]\
                        +subs_wood[i_job,:,:] \
                        +subs_energy[i_job,:,:]

                elif (i_withinyear > i_year): 
                    # Each year the decay starts one year later
                    resid[i_job,i_year,i_withinyear,:,:] = \
                        resid[i_job,i_year,i_withinyear-1,:,:]-\
                        (resid[i_job,i_year,i_withinyear-1,:,:]* \
                             R_decay[i_withinyear-i_year])
           
                else:
                    # A cell in matrix for which there is no
                    # decay
                    resid[i_job,i_year,i_withinyear,:,:] = 0
                    
                # endif

            # endfor i_withinyear

            # We now have a matrix with the residual atmospheric
            # CO2 for each year of emissions. To calculate the
            # currently remaining atmospheric CO2 we have to sum
            # over the last time step. Summing over the previous
            # time steps will give the atmospheric CO2 in that 
            # year.
            atm_co2[i_job,:,:] = np.sum(resid[i_job,:,i_year,:,:],axis=0)
            
        # enfor i_job

        #--------------------------------------------------------------
        # Write netcdf for LCA analysis
        #--------------------------------------------------------------
        filename = jobname[0]+ '_' + str(i_year+year_first)+'0101_'+ str(i_year+year_first)+ '1231_1Y_Emission_history.nc'
        #filename = suffix+ str(i_year) + 'Emission_history.nc'
        ncfile = Dataset(filename,'w') 
        ncfile.createDimension('latitude',n_lat)
        ncfile.createDimension('longitude',n_lon)
        ncfile.createDimension('time',n_years-1)
        ncfile.createDimension('job',n_jobs)
        
        lats = ncfile.createVariable('latitude','f',('latitude',))
        lons = ncfile.createVariable('longitude','f',('longitude',))
        years = ncfile.createVariable('time','i',('time',))
        jobs = ncfile.createVariable('job','i',('job',))
        lats.axis = 'Y'
        lats.units = 'degrees north'
        lons.axis = 'X'
        lons.units = 'degrees east'
        years.units = 'year'
        jobs.units = 'treatment'
        
        # Give values to the dimensions
        lats[:] = lat_sim
        lons[:] = lon_sim 
        # We need to accumulate two years to calculate one nbp value
        # therefore the counters for years are becoming a bit messy!
        for i_year in range(1,n_years):
            years[i_year-1] = year_first + i_year
        # endfor
        for i_job in range(0,n_jobs):
            jobs[i_job] = i_job+1
        # endfor
                
        value1 = ncfile.createVariable('emissions','f',\
           ('job','latitude','longitude'))
        value1.units = 'gC'
        value2 = ncfile.createVariable('nep','f',\
           ('job','latitude','longitude'))
        value2.units = 'gC'
        value3 = ncfile.createVariable('woodsubs','f',\
           ('job','latitude','longitude'))
        value3.units = 'gC'
        value4 = ncfile.createVariable('energysubs','f',\
           ('job','latitude','longitude'))
        value4.units = 'gC'
        value5 = ncfile.createVariable('netatm','f',\
           ('job','latitude','longitude'))
        value5.units = 'gC'
        value6 = ncfile.createVariable('atmco2','f',\
           ('job','latitude','longitude'))
        value6.units = 'gC'

        for i_job in range(0,n_jobs):
        
            for i_lat in range(0,n_lat):

                for i_lon in range(0,n_lon):

                    if (nep[i_job,i_lat,i_lon]==0):
                        value1[i_job,i_lat,i_lon] = float('NaN')
                        value2[i_job,i_lat,i_lon] = float('NaN')
                        value3[i_job,i_lat,i_lon] = float('NaN')
                        value4[i_job,i_lat,i_lon] = float('NaN')
                        value5[i_job,i_lat,i_lon] = float('NaN')         
                        value6[i_job,i_lat,i_lon] = float('NaN')
                        
                    else:
                        value1[i_job,i_lat,i_lon] = \
                            emission_fm[i_job,i_lat,i_lon]
                        value2[i_job,i_lat,i_lon] = \
                            nep[i_job,i_lat,i_lon]
                        value3[i_job,i_lat,i_lon] = \
                            subs_wood[i_job,i_lat,i_lon]
                        value4[i_job,i_lat,i_lon] = \
                            subs_energy[i_job,i_lat,i_lon]
                        # Gross LCA
                        value5[i_job,i_lat,i_lon] = \
                            value1[i_job,i_lat,i_lon]\
                            + value2[i_job,i_lat,i_lon]\
                            + value3[i_job,i_lat,i_lon]\
                            + value4[i_job,i_lat,i_lon]
                        # Net LCA - corrected for ocean uptake
                        value6[i_job,i_lat,i_lon] =\
                            atm_co2[i_job,i_lat,i_lon]
    
                    # endif

                # endfor lon

            # endfor lat        
            
        # endfor i_job
        ncfile.close()

    # Save variables
    #store = shelve.open('archived_maps','n')
    #store['archive'] = fm_strat_map
    #store.close()

# endif recalculate

print 'End of script'
